#!/bin/bash

CLX_ROOT=${CLX_ROOT:-"/opt/mellanox/collectx"}
PLUGIN_RUNNER="${CLX_ROOT}/bin/plugin_runner"
HCAPERF="hcaperf"
HCAPERF_SO_LIBRARY="${CLX_ROOT}/lib/providers/libhcaperf_provider.so"
HCAS_TO_SAMPLE="mlx5_0"
RUNTIME_DIR_ROOT="/tmp/plugin_runner"

# plugin runner parameters
SO_LIBRARY="${HCAPERF_SO_LIBRARY}"
NUM_ITERATIONS=10000
SAMPLING_INTERVAL_MS=1
CSET_DIR="${RUNTIME_DIR_ROOT}/csets/${HCAPERF}"
OUTPUT_DIR="${RUNTIME_DIR_ROOT}/data"
ADDITIONAL_OPTIONS=""
OUTPUT_ENABLED="true"


error() {
  echo "ERROR: $*" 1>&2;
  exit 1
}

print_usage() {
    cat << EOF
Usage: $0 [Optional Parameters]	

    Optional Parameters:
        -c|--cset DIRECTORY                  Directory with cset files for live view    default: ${CSET_DIR}
        -o|--output DIRECTORY                Directory of collected data output         default: ${OUTPUT_DIR}
        -u|--no-output                       Do not write data to output directory      default: false
        -a|--hca HCA_1,...,HCA_N             HCAs to sample (for hcaperf provider)      default: ${HCAS_TO_SAMPLE}
        -n|--num-iterations NUM-ITERATIONS   Number of iterations to run                default: ${NUM_ITERATIONS}
        -s|--sample-time SAMPLE-TIME         Sample time (milliseconds)                 default: ${SAMPLING_INTERVAL_MS}
        -h|--help                            Show this help message and exit
EOF
}

error_bad_key() {
    print_usage
    error "Bad ${KEY}"
}

set_default_cset_dir() {
    [[ -d "${CSET_DIR}" ]] && return
    mkdir -p "${CSET_DIR}" || error "cannot make cset directory ${CSET_DIR}"
    cat << EOF > "${CSET_DIR}/${HCAPERF}.cset"
# substrings to include
+rx
+pxt
+sxp
+tx
+sx
+tpt
+pci

# suffixes to exclude
-_min$
-_max$
-_ref$
-_grade$
-_util$
EOF
}

unpack_hcas() {
    hcas_to_sample="${1}"
    IFS=',' read -r -a hcas <<< "$hcas_to_sample"
    for ca in "${hcas[@]}"
    do
        ADDITIONAL_OPTIONS+="-a hcaperf.${ca}=sample "
    done
}

set_default_cset_dir

while [ $# -gt 0 ]; do
KEY="$1"

  case $KEY in
    -c|--cset)
                    CSET_DIR="${2}"
                    [ -z "${CSET_DIR}" ] && error_bad_key
                    shift
    ;;
    -o|--output)
                    OUTPUT_DIR="${2}"
                    [ -z "${OUTPUT_DIR}" ] && error_bad_key
                    shift
    ;;
    -u|--no-output)
                    OUTPUT_ENABLED="false"
    ;;
    -a|--hca)
                    HCAS_TO_SAMPLE="${2}"
                    [ -z "${HCAS_TO_SAMPLE}" ] && error_bad_key
                    unpack_hcas "${HCAS_TO_SAMPLE}"
                    shift
    ;;
    -n|--num-iterations)
                    NUM_ITERATIONS="${2}"
                    [ -z "${NUM_ITERATIONS}" ] && error_bad_key
                    shift
    ;;
    -s|--sample-time)
                    SAMPLING_INTERVAL_MS="${2}"
                    [ -z "${SAMPLING_INTERVAL_MS}" ] && error_bad_key
                    shift
    ;;
    -h|--help)      print_usage
                    exit 2
    ;;
    *)              # unknown option
                    print_usage
                    error "Unknown parameter: $key"
    ;;
  esac
shift
done

[[ -z "${ADDITIONAL_OPTIONS}" ]] && unpack_hcas "${HCAS_TO_SAMPLE}"

if [[ "${OUTPUT_ENABLED}" == true ]]; then
  [[ ! -d "${OUTPUT_DIR}" ]] && mkdir -p "${OUTPUT_DIR}"
  ADDITIONAL_OPTIONS+="--data-root ${OUTPUT_DIR} "
else
  ADDITIONAL_OPTIONS+="--disable-file-writer "
fi

$PLUGIN_RUNNER --so-name "${SO_LIBRARY}"                        \
               --num-iters "${NUM_ITERATIONS}"                  \
               --sampling-interval-ms "${SAMPLING_INTERVAL_MS}" \
               --view-folder "${CSET_DIR}"                      \
               ${ADDITIONAL_OPTIONS}
